// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
'use strict';
var console_1 = require('jupyter-js-notebook/lib/console');
var rendermime_1 = require('jupyter-js-ui/lib/rendermime');
var docmanager_1 = require('jupyter-js-ui/lib/docmanager');
var phosphor_tabs_1 = require('phosphor-tabs');
var plugin_1 = require('../services/plugin');
/**
 * The console extension.
 */
exports.consoleExtension = {
    id: 'jupyter.extensions.console',
    requires: [plugin_1.JupyterServices, rendermime_1.RenderMime],
    activate: activateConsole
};
/**
 * Activate the console extension.
 */
function activateConsole(app, services, rendermime) {
    var manager = services.notebookSessionManager;
    // Add the ability to create new consoles for each kernel.
    var specs = services.kernelspecs;
    var displayNameMap = Object.create(null);
    for (var kernelName in specs.kernelspecs) {
        var displayName = specs.kernelspecs[kernelName].spec.display_name;
        displayNameMap[displayName] = kernelName;
    }
    var displayNames = Object.keys(displayNameMap).sort(function (a, b) {
        return a.localeCompare(b);
    });
    var count = 0;
    var _loop_1 = function(displayName) {
        var id = "console:create-" + displayNameMap[displayName];
        app.commands.add([{
                id: id,
                handler: function () {
                    manager.startNew({
                        notebookPath: "Console-" + count++,
                        kernelName: "" + displayNameMap[displayName]
                    }).then(function (session) {
                        var console = new console_1.ConsolePanel(session, rendermime.clone());
                        console.id = "console-" + count;
                        console.title.text = "Console-" + count;
                        console.title.closable = true;
                        app.shell.addToMainArea(console);
                        Private.activeWidget = console;
                        // TODO: Move this logic to the shell.
                        var stack = console.parent;
                        if (!stack) {
                            return;
                        }
                        var tabs = stack.parent;
                        if (tabs instanceof phosphor_tabs_1.TabPanel) {
                            tabs.currentWidget = console;
                        }
                        console.content.prompt.focus();
                        console.disposed.connect(function () {
                            var index = Private.widgets.indexOf(console);
                            Private.widgets.splice(index, 1);
                            if (Private.activeWidget === console) {
                                Private.activeWidget = null;
                            }
                        });
                    });
                }
            }]);
        app.palette.add([{
                command: id,
                category: 'Console',
                text: "New " + displayName
            }]);
    };
    for (var _i = 0, displayNames_1 = displayNames; _i < displayNames_1.length; _i++) {
        var displayName = displayNames_1[_i];
        _loop_1(displayName);
    }
    // Temporary console focus follower.
    document.body.addEventListener('focus', function (event) {
        for (var _i = 0, _a = Private.widgets; _i < _a.length; _i++) {
            var widget = _a[_i];
            var target = event.target;
            if (widget.isAttached && widget.isVisible) {
                if (widget.node.contains(target)) {
                    Private.activeWidget = widget;
                    return;
                }
            }
        }
    }, true);
    app.commands.add([
        {
            id: 'console:clear',
            handler: function () {
                if (Private.activeWidget) {
                    Private.activeWidget.content.clear();
                }
            }
        },
        {
            id: 'console:execute',
            handler: function () {
                if (Private.activeWidget) {
                    Private.activeWidget.content.execute();
                }
            }
        },
        {
            id: 'console:interrupt-kernel',
            handler: function () {
                if (Private.activeWidget) {
                    var kernel = Private.activeWidget.content.session.kernel;
                    if (kernel) {
                        kernel.interrupt();
                    }
                }
            }
        },
        {
            id: 'console:switch-kernel',
            handler: function () {
                if (Private.activeWidget) {
                    var widget_1 = Private.activeWidget.content;
                    var session_1 = widget_1.session;
                    var lang_1 = '';
                    if (session_1.kernel) {
                        lang_1 = specs.kernelspecs[session_1.kernel.name].spec.language;
                    }
                    manager.listRunning().then(function (sessions) {
                        var options = {
                            name: widget_1.parent.title.text,
                            specs: specs,
                            sessions: sessions,
                            preferredLanguage: lang_1,
                            kernel: session_1.kernel,
                            host: widget_1.parent.node
                        };
                        return docmanager_1.selectKernel(options);
                    }).then(function (kernelId) {
                        if (kernelId) {
                            session_1.changeKernel(kernelId);
                        }
                        else {
                            session_1.kernel.shutdown();
                        }
                    });
                }
            }
        }
    ]);
    app.palette.add([
        {
            command: 'console:clear',
            category: 'Console',
            text: 'Clear Cells'
        },
        {
            command: 'console:execute',
            category: 'Console',
            text: 'Execute Cell'
        },
        {
            command: 'console:interrupt-kernel',
            category: 'Console',
            text: 'Interrupt Kernel'
        },
        {
            command: 'console:switch-kernel',
            category: 'Console',
            text: 'Switch Kernel'
        }]);
    return Promise.resolve(void 0);
}
/**
 * A namespace for private data.
 */
var Private;
(function (Private) {
    Private.widgets = [];
    Private.activeWidget = null;
})(Private || (Private = {}));
