// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
'use strict';
var filebrowser_1 = require('jupyter-js-ui/lib/filebrowser');
var docmanager_1 = require('jupyter-js-ui/lib/docmanager');
var phosphor_menus_1 = require('phosphor-menus');
var phosphor_tabs_1 = require('phosphor-tabs');
var plugin_1 = require('../services/plugin');
/**
 * The default file browser extension.
 */
exports.fileBrowserExtension = {
    id: 'jupyter.extensions.fileBrowser',
    requires: [plugin_1.JupyterServices, docmanager_1.DocumentRegistry],
    activate: activateFileBrowser
};
/**
 * Activate the file browser.
 */
function activateFileBrowser(app, provider, registry) {
    var contents = provider.contentsManager;
    var sessions = provider.notebookSessionManager;
    var widgets = [];
    var activeWidget;
    var id = 0;
    var opener = {
        open: function (widget) {
            if (!widget.id) {
                widget.id = "document-manager-" + ++id;
            }
            if (!widget.isAttached) {
                app.shell.addToMainArea(widget);
            }
            // TODO: Move this logic to the shell.
            var stack = widget.parent;
            if (!stack) {
                return;
            }
            var tabs = stack.parent;
            if (tabs instanceof phosphor_tabs_1.TabPanel) {
                tabs.currentWidget = widget;
            }
            activeWidget = widget;
            widget.disposed.connect(function (w) {
                var index = widgets.indexOf(w);
                widgets.splice(index, 1);
            });
        }
    };
    // TODO: Move focus tracking to the shell.
    document.addEventListener('focus', function (event) {
        for (var i = 0; i < widgets.length; i++) {
            var widget_1 = widgets[i];
            if (widget_1.node.contains(event.target)) {
                activeWidget = widget_1;
                break;
            }
        }
    });
    var docManager = new docmanager_1.DocumentManager(registry, contents, sessions, provider.kernelspecs, opener);
    var model = new filebrowser_1.FileBrowserModel(contents, sessions, provider.kernelspecs);
    var widget = new filebrowser_1.FileBrowserWidget(model, docManager, opener);
    var menu = createMenu(widget);
    // Add a context menu to the dir listing.
    var node = widget.node.getElementsByClassName('jp-DirListing-content')[0];
    node.addEventListener('contextmenu', function (event) {
        event.preventDefault();
        var x = event.clientX;
        var y = event.clientY;
        menu.popup(x, y);
    });
    // Add the command for a new items.
    var newTextFileId = 'file-operations:new-text-file';
    app.commands.add([
        {
            id: newTextFileId,
            handler: function () { return widget.createNew('file'); }
        }
    ]);
    var newNotebookId = 'file-operations:new-notebook';
    app.commands.add([
        {
            id: newNotebookId,
            handler: function () { return widget.createNew('notebook'); }
        }]);
    // Add the command for saving a document.
    var saveDocumentId = 'file-operations:save';
    app.commands.add([
        {
            id: saveDocumentId,
            handler: function () {
                if (!activeWidget) {
                    return;
                }
                activeWidget.context.save();
            }
        }
    ]);
    app.palette.add([
        {
            command: saveDocumentId,
            category: 'File Operations',
            text: 'Save Document',
            caption: 'Save the current document'
        }
    ]);
    // Add the command for reverting a document.
    var revertDocumentId = 'file-operations:revert';
    app.commands.add([
        {
            id: revertDocumentId,
            handler: function () {
                if (!activeWidget) {
                    return;
                }
                activeWidget.context.revert();
            }
        }
    ]);
    app.palette.add([
        {
            command: revertDocumentId,
            category: 'File Operations',
            text: 'Revert Document',
            caption: 'Revert the current document'
        }
    ]);
    // Add the command for closing a document.
    var closeDocumentId = 'file-operations:close';
    app.commands.add([
        {
            id: closeDocumentId,
            handler: function () {
                if (activeWidget) {
                    activeWidget.close();
                }
            }
        }
    ]);
    app.palette.add([
        {
            command: closeDocumentId,
            category: 'File Operations',
            text: 'Close Document',
            caption: 'Close the current document'
        }
    ]);
    // Add the command for closing all documents.
    var closeAllId = 'file-operations:close-all';
    app.commands.add([
        {
            id: closeAllId,
            handler: function () {
                docManager.closeAll();
            }
        }
    ]);
    app.palette.add([
        {
            command: closeAllId,
            category: 'File Operations',
            text: 'Close All',
            caption: 'Close all open documents'
        }
    ]);
    app.palette.add([
        {
            command: newTextFileId,
            category: 'File Operations',
            text: 'New Text File',
            caption: 'Create a new text file'
        },
        {
            command: newNotebookId,
            category: 'File Operations',
            text: 'New Notebook',
            caption: 'Create a new notebook'
        }
    ]);
    app.commands.add([
        {
            id: 'file-browser:activate',
            handler: showBrowser
        },
        {
            id: 'file-browser:hide',
            handler: hideBrowser
        },
        {
            id: 'file-browser:toggle',
            handler: toggleBrowser
        }
    ]);
    widget.title.text = 'Files';
    widget.id = 'file-browser';
    app.shell.addToLeftArea(widget, { rank: 40 });
    showBrowser();
    return Promise.resolve(void 0);
    function showBrowser() {
        app.shell.activateLeft(widget.id);
    }
    function hideBrowser() {
        if (!widget.isHidden) {
            app.shell.collapseLeft();
        }
    }
    function toggleBrowser() {
        if (widget.isHidden) {
            showBrowser();
        }
        else {
            hideBrowser();
        }
    }
}
/**
 * Create a context menu for the file browser listing.
 */
function createMenu(fbWidget) {
    return new phosphor_menus_1.Menu([
        new phosphor_menus_1.MenuItem({
            text: '&Open',
            icon: 'fa fa-folder-open-o',
            shortcut: 'Ctrl+O',
            handler: function () { fbWidget.open(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: '&Rename',
            icon: 'fa fa-edit',
            shortcut: 'Ctrl+R',
            handler: function () { fbWidget.rename(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: '&Delete',
            icon: 'fa fa-remove',
            shortcut: 'Ctrl+D',
            handler: function () { fbWidget.delete(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: 'Duplicate',
            icon: 'fa fa-copy',
            handler: function () { fbWidget.duplicate(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: 'Cut',
            icon: 'fa fa-cut',
            shortcut: 'Ctrl+X',
            handler: function () { fbWidget.cut(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: '&Copy',
            icon: 'fa fa-copy',
            shortcut: 'Ctrl+C',
            handler: function () { fbWidget.copy(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: '&Paste',
            icon: 'fa fa-paste',
            shortcut: 'Ctrl+V',
            handler: function () { fbWidget.paste(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: 'Download',
            icon: 'fa fa-download',
            handler: function () { fbWidget.download(); }
        }),
        new phosphor_menus_1.MenuItem({
            text: 'Shutdown Kernel',
            icon: 'fa fa-stop-circle-o',
            handler: function () { fbWidget.shutdownKernels(); }
        })
    ]);
}
