// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
'use strict';
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var jupyter_js_notebook_1 = require('jupyter-js-notebook');
var docmanager_1 = require('jupyter-js-ui/lib/docmanager');
var rendermime_1 = require('jupyter-js-ui/lib/rendermime');
var phosphor_dragdrop_1 = require('phosphor-dragdrop');
var phosphor_signaling_1 = require('phosphor-signaling');
var plugin_1 = require('../services/plugin');
/**
 * The map of command ids used by the notebook.
 */
var cmdIds = {
    interrupt: 'notebook:interrupt-kernel',
    restart: 'notebook:restart-kernel',
    restartClear: 'notebook:restart-clear',
    restartRunAll: 'notebook:restart-runAll',
    switchKernel: 'notebook:switch-kernel',
    clearAllOutputs: 'notebook:clear-outputs',
    run: 'notebook-cells:run',
    runAndAdvance: 'notebook-cells:runAndAdvance',
    runAndInsert: 'notebook-cells:runAndInsert',
    runAll: 'notebook:run-all',
    toCode: 'notebook-cells:to-code',
    toMarkdown: 'notebook-cells:to-markdown',
    toRaw: 'notebook-cells:to-raw',
    cut: 'notebook-cells:cut',
    copy: 'notebook-cells:copy',
    paste: 'notebook-cells:paste',
    clearOutputs: 'notebook-cells:clear-output',
    deleteCell: 'notebook-cells:delete',
    insertAbove: 'notebook-cells:insert-above',
    insertBelow: 'notebook-cells:insert-below',
    selectAbove: 'notebook-cells:select-above',
    selectBelow: 'notebook-cells:select-below',
    extendAbove: 'notebook-cells:extend-above',
    extendBelow: 'notebook-cells:extend-below',
    editMode: 'notebook:editMode',
    merge: 'notebook-cells:merge',
    split: 'notebook-cells:split',
    commandMode: 'notebook:commandMode',
    newNotebook: 'notebook:create-new',
    toggleLines: 'notebook-cells:toggle-lineNumbers',
    toggleAllLines: 'notebook-cells:toggle-allLineNumbers',
    undo: 'notebook-cells:undo',
    redo: 'notebook-cells:redo'
};
/**
 * The notebook file handler provider.
 */
exports.notebookHandlerExtension = {
    id: 'jupyter.extensions.notebookHandler',
    requires: [docmanager_1.DocumentRegistry, plugin_1.JupyterServices, rendermime_1.RenderMime, phosphor_dragdrop_1.MimeData],
    activate: activateNotebookHandler
};
/**
 * An interface exposing the current active notebook.
 */
var ActiveNotebook = (function () {
    /**
     * Construct a new active notebook tracker.
     */
    function ActiveNotebook() {
        var _this = this;
        this._activeWidget = null;
        this._widgets = [];
        // Temporary notebook focus follower.
        document.body.addEventListener('focus', function (event) {
            for (var _i = 0, _a = _this._widgets; _i < _a.length; _i++) {
                var widget = _a[_i];
                var target = event.target;
                if (widget.isAttached && widget.isVisible) {
                    if (widget.node.contains(target)) {
                        _this.activeNotebook = widget;
                        return;
                    }
                }
            }
        }, true);
    }
    Object.defineProperty(ActiveNotebook.prototype, "activeNotebookChanged", {
        /**
         * A signal emitted when the active notebook changes.
         */
        get: function () {
            return Private.activeNotebookChangedSignal.bind(this);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActiveNotebook.prototype, "activeNotebook", {
        /**
         * The current active notebook.
         */
        get: function () {
            return this._activeWidget;
        },
        set: function (widget) {
            var _this = this;
            if (this._activeWidget === widget) {
                return;
            }
            if (this._widgets.indexOf(widget) !== -1) {
                this._activeWidget = widget;
                this.activeNotebookChanged.emit(widget);
                return;
            }
            if (widget === null) {
                return;
            }
            this._widgets.push(widget);
            widget.disposed.connect(function () {
                var index = _this._widgets.indexOf(widget);
                _this._widgets.splice(index, 1);
                if (_this._activeWidget === widget) {
                    _this.activeNotebook = null;
                }
            });
        },
        enumerable: true,
        configurable: true
    });
    return ActiveNotebook;
}());
exports.ActiveNotebook = ActiveNotebook;
/**
 * A service tracking the active notebook widget.
 */
exports.activeNotebookProvider = {
    id: 'jupyter.services.activeNotebook',
    provides: ActiveNotebook,
    resolve: function () {
        return Private.notebookTracker;
    }
};
/**
 * A version of the notebook widget factory that uses the notebook tracker.
 */
var TrackingNotebookWidgetFactory = (function (_super) {
    __extends(TrackingNotebookWidgetFactory, _super);
    function TrackingNotebookWidgetFactory() {
        _super.apply(this, arguments);
    }
    /**
     * Create a new widget.
     */
    TrackingNotebookWidgetFactory.prototype.createNew = function (model, context, kernel) {
        var widget = _super.prototype.createNew.call(this, model, context, kernel);
        Private.notebookTracker.activeNotebook = widget;
        return widget;
    };
    return TrackingNotebookWidgetFactory;
}(jupyter_js_notebook_1.NotebookWidgetFactory));
/**
 * Activate the notebook handler extension.
 */
function activateNotebookHandler(app, registry, services, rendermime, clipboard) {
    var widgetFactory = new TrackingNotebookWidgetFactory(rendermime, clipboard);
    registry.registerModelFactory(new jupyter_js_notebook_1.NotebookModelFactory());
    registry.registerWidgetFactory(widgetFactory, {
        fileExtensions: ['.ipynb'],
        displayName: 'Notebook',
        modelName: 'notebook',
        defaultFor: ['.ipynb'],
        preferKernel: true,
        canStartKernel: true
    });
    // Add the ability to launch notebooks for each kernel type.
    var displayNameMap = Object.create(null);
    var specs = services.kernelspecs;
    for (var kernelName in specs.kernelspecs) {
        var displayName = specs.kernelspecs[kernelName].spec.display_name;
        displayNameMap[displayName] = kernelName;
    }
    var displayNames = Object.keys(displayNameMap).sort(function (a, b) {
        return a.localeCompare(b);
    });
    for (var _i = 0, displayNames_1 = displayNames; _i < displayNames_1.length; _i++) {
        var displayName = displayNames_1[_i];
        registry.registerCreator({
            name: displayName + " Notebook",
            extension: '.ipynb',
            type: 'notebook',
            kernelName: displayNameMap[displayName]
        });
    }
    var tracker = Private.notebookTracker;
    app.commands.add([
        {
            id: cmdIds['runAndAdvance'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.runAndAdvance(nbWidget.content, nbWidget.context.kernel);
                }
            }
        },
        {
            id: cmdIds['run'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.run(nbWidget.content, nbWidget.context.kernel);
                }
            }
        },
        {
            id: cmdIds['runAndInsert'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.runAndInsert(nbWidget.content, nbWidget.context.kernel);
                }
            }
        },
        {
            id: cmdIds['runAll'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.runAll(nbWidget.content, nbWidget.context.kernel);
                }
            }
        },
        {
            id: cmdIds['restart'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    nbWidget.restart();
                }
            }
        },
        {
            id: cmdIds['restartClear'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget_1 = tracker.activeNotebook;
                    nbWidget_1.restart().then(function (result) {
                        if (result) {
                            jupyter_js_notebook_1.NotebookActions.clearAllOutputs(nbWidget_1.content);
                        }
                    });
                }
            }
        },
        {
            id: cmdIds['restartRunAll'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget_2 = tracker.activeNotebook;
                    nbWidget_2.restart().then(function (result) {
                        jupyter_js_notebook_1.NotebookActions.runAll(nbWidget_2.content, nbWidget_2.context.kernel);
                    });
                }
            }
        },
        {
            id: cmdIds['clearAllOutputs'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.clearAllOutputs(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['clearOutputs'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.clearOutputs(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['interrupt'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var kernel = tracker.activeNotebook.context.kernel;
                    if (kernel) {
                        kernel.interrupt();
                    }
                }
            }
        },
        {
            id: cmdIds['toCode'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.changeCellType(nbWidget.content, 'code');
                }
            }
        },
        {
            id: cmdIds['toMarkdown'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.changeCellType(nbWidget.content, 'markdown');
                }
            }
        },
        {
            id: cmdIds['toRaw'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.changeCellType(nbWidget.content, 'raw');
                }
            }
        },
        {
            id: cmdIds['cut'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.cut(nbWidget.content, nbWidget.clipboard);
                }
            }
        },
        {
            id: cmdIds['copy'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.copy(nbWidget.content, nbWidget.clipboard);
                }
            }
        },
        {
            id: cmdIds['paste'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.paste(nbWidget.content, nbWidget.clipboard);
                }
            }
        },
        {
            id: cmdIds['deleteCell'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.deleteCells(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['split'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.splitCell(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['merge'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.mergeCells(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['insertAbove'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.insertAbove(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['insertBelow'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.insertBelow(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['selectAbove'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.selectAbove(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['selectBelow'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.selectBelow(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['extendAbove'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.extendSelectionAbove(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['extendBelow'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.extendSelectionBelow(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['toggleLines'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.toggleLineNumbers(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['toggleAllLines'],
            handler: function () {
                if (tracker.activeNotebook) {
                    var nbWidget = tracker.activeNotebook;
                    jupyter_js_notebook_1.NotebookActions.toggleAllLineNumbers(nbWidget.content);
                }
            }
        },
        {
            id: cmdIds['commandMode'],
            handler: function () {
                if (tracker.activeNotebook) {
                    tracker.activeNotebook.content.mode = 'command';
                }
            }
        },
        {
            id: cmdIds['editMode'],
            handler: function () {
                if (tracker.activeNotebook) {
                    tracker.activeNotebook.content.mode = 'edit';
                }
            }
        },
        {
            id: cmdIds['undo'],
            handler: function () {
                if (tracker.activeNotebook) {
                    jupyter_js_notebook_1.NotebookActions.undo(tracker.activeNotebook.content);
                }
            }
        },
        {
            id: cmdIds['redo'],
            handler: function () {
                if (tracker.activeNotebook) {
                    jupyter_js_notebook_1.NotebookActions.redo(tracker.activeNotebook.content);
                }
            }
        },
        {
            id: cmdIds['switchKernel'],
            handler: function () {
                if (tracker.activeNotebook) {
                    docmanager_1.selectKernelForContext(tracker.activeNotebook.context, tracker.activeNotebook.node);
                }
            }
        }
    ]);
    app.palette.add([
        {
            command: cmdIds['run'],
            category: 'Notebook Cell Operations',
            text: 'Run Cell(s)'
        },
        {
            command: cmdIds['runAndAdvance'],
            category: 'Notebook Cell Operations',
            text: 'Run Cell(s) and Advance'
        },
        {
            command: cmdIds['runAndInsert'],
            category: 'Notebook Cell Operations',
            text: 'Run Cell(s) and Insert'
        },
        {
            command: cmdIds['interrupt'],
            category: 'Notebook Operations',
            text: 'Interrupt Kernel'
        },
        {
            command: cmdIds['restart'],
            category: 'Notebook Operations',
            text: 'Restart Kernel'
        },
        {
            command: cmdIds['restartClear'],
            category: 'Notebook Operations',
            text: 'Restart Kernel & Clear Outputs'
        },
        {
            command: cmdIds['restartRunAll'],
            category: 'Notebook Operations',
            text: 'Restart Kernel & Run All'
        },
        {
            command: cmdIds['runAll'],
            category: 'Notebook Operations',
            text: 'Run All Cells'
        },
        {
            command: cmdIds['clearAllOutputs'],
            category: 'Notebook Operations',
            text: 'Clear All Outputs'
        },
        {
            command: cmdIds['clearOutputs'],
            category: 'Notebook Cell Operations',
            text: 'Clear Output(s)'
        },
        {
            command: cmdIds['toCode'],
            category: 'Notebook Cell Operations',
            text: 'Convert to Code'
        },
        {
            command: cmdIds['toMarkdown'],
            category: 'Notebook Cell Operations',
            text: 'Convert to Markdown'
        },
        {
            command: cmdIds['toRaw'],
            category: 'Notebook Cell Operations',
            text: 'Convert to Raw'
        },
        {
            command: cmdIds['cut'],
            category: 'Notebook Cell Operations',
            text: 'Cut Cell(s)'
        },
        {
            command: cmdIds['copy'],
            category: 'Notebook Cell Operations',
            text: 'Copy Cell(s)'
        },
        {
            command: cmdIds['paste'],
            category: 'Notebook Cell Operations',
            text: 'Paste Cell(s)'
        },
        {
            command: cmdIds['deleteCell'],
            category: 'Notebook Cell Operations',
            text: 'Delete Cell(s)'
        },
        {
            command: cmdIds['split'],
            category: 'Notebook Cell Operations',
            text: 'Split Cell'
        },
        {
            command: cmdIds['merge'],
            category: 'Notebook Cell Operations',
            text: 'Merge Selected Cell(s)'
        },
        {
            command: cmdIds['insertAbove'],
            category: 'Notebook Cell Operations',
            text: 'Insert Cell Above'
        },
        {
            command: cmdIds['insertBelow'],
            category: 'Notebook Cell Operations',
            text: 'Insert Cell Below'
        },
        {
            command: cmdIds['selectAbove'],
            category: 'Notebook Cell Operations',
            text: 'Select Cell Above'
        },
        {
            command: cmdIds['selectBelow'],
            category: 'Notebook Cell Operations',
            text: 'Select Cell Below'
        },
        {
            command: cmdIds['extendAbove'],
            category: 'Notebook Cell Operations',
            text: 'Extend Selection Above'
        },
        {
            command: cmdIds['extendBelow'],
            category: 'Notebook Cell Operations',
            text: 'Extend Selection Below'
        },
        {
            command: cmdIds['toggleLines'],
            category: 'Notebook Cell Operations',
            text: 'Toggle Line Numbers'
        },
        {
            command: cmdIds['toggleAllLines'],
            category: 'Notebook Operations',
            text: 'Toggle All Line Numbers'
        },
        {
            command: cmdIds['editMode'],
            category: 'Notebook Operations',
            text: 'To Edit Mode'
        },
        {
            command: cmdIds['commandMode'],
            category: 'Notebook Operations',
            text: 'To Command Mode'
        },
        {
            command: cmdIds['switchKernel'],
            category: 'Notebook Operations',
            text: 'Switch Kernel'
        },
        {
            command: cmdIds['undo'],
            category: 'Notebook Cell Operations',
            text: 'Undo Cell Operation'
        },
        {
            command: cmdIds['redo'],
            category: 'Notebook Cell Operations',
            text: 'Redo Cell Operation'
        }
    ]);
    return Promise.resolve(void 0);
}
/**
 * A namespace for notebook plugin private data.
 */
var Private;
(function (Private) {
    /**
     * A signal emitted when the active notebook changes.
     */
    Private.activeNotebookChangedSignal = new phosphor_signaling_1.Signal();
    /**
     * A singleton notebook tracker instance.
     */
    Private.notebookTracker = new ActiveNotebook();
})(Private || (Private = {}));
